var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useState, useRef, useEffect } from "react";
import { rehype } from "rehype";
import { selectAll } from "hast-util-select";
import { remove } from "unist-util-remove";
function createBase64ImageStringFromURL(url) {
    return new Promise((resolve, reject) => {
        fetch(url)
            .then((response) => {
            if (!response.ok) {
                reject("Unable to resolve image path");
            }
            return response.blob();
        })
            .then((blob) => {
            const reader = new FileReader();
            reader.addEventListener("load", () => {
                if (typeof reader.result !== "string") {
                    return reject("File reader result value should be of type string");
                }
                resolve(reader.result);
            });
            reader.readAsDataURL(blob);
        });
    });
}
function rehypeInlineImages() {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        const images = selectAll("img", tree);
        yield Promise.all(images.map((image) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const imagePath = (_a = image === null || image === void 0 ? void 0 : image.properties) === null || _a === void 0 ? void 0 : _a.src;
            if (!imagePath ||
                typeof imagePath !== "string" ||
                imagePath.startsWith("data:")) {
                return;
            }
            const base64image = yield createBase64ImageStringFromURL(imagePath);
            if (base64image) {
                image.properties = Object.assign(Object.assign({}, image.properties), { src: base64image, id: "test" });
            }
        })));
    });
}
function rehypeStripSVGs() {
    return (tree) => {
        remove(tree, (node) => {
            return node.type === "element" && node.tagName === "svg";
        });
    };
}
function prepareHTMLforPrint(html) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield rehype()
            .use(rehypeStripSVGs)
            .use(rehypeInlineImages)
            .process(html);
        return file.value;
    });
}
const initialState = {
    idle: false,
    loading: false,
    success: false,
    error: false,
};
function useHtmlToPdfUtil(key, endpoint) {
    const [status, setStatus] = useState(initialState);
    const cachedHTML = useRef();
    const downloadURL = useRef();
    function purgeObjectURL() {
        var _a;
        URL.revokeObjectURL((_a = downloadURL === null || downloadURL === void 0 ? void 0 : downloadURL.current) !== null && _a !== void 0 ? _a : "");
    }
    useEffect(() => {
        return () => {
            purgeObjectURL();
            setStatus(initialState);
        };
    }, [key]);
    const print = (html, css) => __awaiter(this, void 0, void 0, function* () {
        try {
            setStatus((prevStatus) => (Object.assign(Object.assign({}, prevStatus), { loading: true })));
            if ((cachedHTML === null || cachedHTML === void 0 ? void 0 : cachedHTML.current) !== html || !(downloadURL === null || downloadURL === void 0 ? void 0 : downloadURL.current)) {
                purgeObjectURL();
                const preparedHTML = yield prepareHTMLforPrint(html);
                const body = JSON.stringify({ html: preparedHTML, css });
                const resp = yield fetch(endpoint, {
                    method: "post",
                    headers: { "Content-Type": "application/json" },
                    body,
                });
                if (!resp.ok) {
                    throw new Error(resp.statusText);
                }
                const blob = yield resp.blob();
                downloadURL.current = URL.createObjectURL(blob);
                cachedHTML.current = html;
            }
            const downloadLink = document.createElement("a");
            downloadLink.href = downloadURL.current;
            downloadLink.download = "document.pdf";
            document.body.appendChild(downloadLink);
            downloadLink.click();
            downloadLink.remove();
            setStatus((prevStatus) => (Object.assign(Object.assign({}, prevStatus), { success: true, error: false })));
        }
        catch (error) {
            console.error(error);
            setStatus((prevStatus) => (Object.assign(Object.assign({}, prevStatus), { success: false, error: true })));
        }
        finally {
            setStatus((prevStatus) => (Object.assign(Object.assign({}, prevStatus), { idle: false, loading: false })));
        }
    });
    return Object.assign({ print }, status);
}
export { useHtmlToPdfUtil };
